@@ -54,6 +54,20 @@ class TargetsTestCase(unittest.TestCase):
         self.assertEqual([cat1], self.targets.get_assessments())
         self.targets.db.add_categories.assert_called_with(assessments)
 
+    def test_build_scope_host_db(self):
+        """Should build a scope that can be ingested into the Database given a Synack API Scope"""
+        scope = [
+            '10.0.0.0/31',
+            '192.168.254.15'
+        ]
+        slug = 'b23iuub'
+        expected = [
+            {'target': slug, 'ip': '10.0.0.0'},
+            {'target': slug, 'ip': '10.0.0.1'},
+            {'target': slug, 'ip': '192.168.254.15'},
+        ]
+        self.assertEqual(expected, self.targets.build_scope_host_db(slug, scope))
+
     def test_build_scope_web_burp(self):
         """Should build a Burp Suite Scope given a Synack API Scope"""
         scope = [
@@ -104,6 +118,36 @@ class TargetsTestCase(unittest.TestCase):
         }
         self.assertEqual(expected, self.targets.build_scope_web_burp(scope))
 
+    def test_build_scope_web_db(self):
+        """Should build a web scope that can be ingested into the Database"""
+        scope = [
+            {
+                'raw_url': 'https://good.stuff.com',
+                'owners': [{'owner_uid': '12345'}, {'owner_uid': '67890'}],
+                'status': 'in'
+            },
+            {
+                'raw_url': 'https://bad.stuff.com',
+                'owners': [{'owner_uid': 'abcde'}],
+                'status': 'out'
+            }
+        ]
+        expected = [
+            {
+                'target': '12345',
+                'urls': [{
+                    'url': 'https://good.stuff.com'
+                }]
+            },
+            {
+                'target': '67890',
+                'urls': [{
+                    'url': 'https://good.stuff.com'
+                }]
+            }
+        ]
+        self.assertEqual(expected, self.targets.build_scope_web_db(scope))
+
     def test_build_scope_web_urls(self):
         """Should build dictionaries of Web Application URLs given a Synack API Scope"""
         scope = [
@@ -171,6 +215,13 @@ class TargetsTestCase(unittest.TestCase):
         self.targets.db.find_targets.assert_has_calls(calls)
         self.targets.get_registered_summary.assert_called_with()
 
+    def test_build_codename_from_slug_invalid(self):
+        """Should return NONE if non-real slug"""
+        self.targets.db.find_targets.return_value = []
+        self.assertEqual("NONE",
+                         self.targets.build_codename_from_slug("qwfars"))
+        self.targets.db.find_targets.assert_called_with(slug="qwfars")
+
     def test_get_connected(self):
         """Should make a request to get the currently selected target"""
         self.targets.api.request.return_value.status_code = 200
@@ -205,6 +256,23 @@ class TargetsTestCase(unittest.TestCase):
         }
         self.assertEqual(out, self.targets.get_connected())
 
+    def test_get_connected_disconnected(self):
+        """Should report Not Connected when not connected to a target"""
+        self.targets.api.request.return_value.status_code = 200
+        self.targets.api.request.return_value.json.return_value = {
+            "pending_slug": "-1",
+            "slug": "",
+            "status": "connected"
+        }
+        self.targets.build_codename_from_slug = MagicMock()
+        self.targets.build_codename_from_slug.return_value = "NONE"
+        out = {
+            "slug": "",
+            "codename": "NONE",
+            "status": "Not Connected"
+        }
+        self.assertEqual(out, self.targets.get_connected())
+
     def test_get_scope_for_host(self):
         """Should get the scope for a Host when given Host information"""
         self.targets.get_scope_host = MagicMock()
@@ -347,6 +415,38 @@ class TargetsTestCase(unittest.TestCase):
                                                     "targets",
                                                     query=query)
 
+    def test_set_connected(self):
+        """Should connect to a given target provided kwargs"""
+        self.targets.db.find_targets.return_value = [Target(slug='28h93iw')]
+        self.targets.api.request.return_value.status_code = 200
+        self.targets.get_connected = MagicMock()
+        self.targets.set_connected(slug='28h93iw')
+        self.targets.api.request.assert_called_with('PUT',
+                                                    'launchpoint',
+                                                    data={'listing_id': '28h93iw'})
+        self.targets.get_connected.assert_called_with()
+
+    def test_set_connected_target(self):
+        """Should connect to a given target provided a target"""
+        target = Target(slug='28h93iw')
+        self.targets.api.request.return_value.status_code = 200
+        self.targets.get_connected = MagicMock()
+        self.targets.set_connected(target)
+        self.targets.api.request.assert_called_with('PUT',
+                                                    'launchpoint',
+                                                    data={'listing_id': '28h93iw'})
+        self.targets.get_connected.assert_called_with()
+
+    def test_set_connected_disconnect(self):
+        """Should disconnect from target if none specified"""
+        self.targets.api.request.return_value.status_code = 200
+        self.targets.get_connected = MagicMock()
+        self.targets.set_connected()
+        self.targets.api.request.assert_called_with('PUT',
+                                                    'launchpoint',
+                                                    data={'listing_id': ''})
+        self.targets.get_connected.assert_called_with()
+
     def test_set_registered(self):
         """Should register each unregistered target"""
         self.targets.get_unregistered = MagicMock()
